SystemOrganization addCategory: #'Control-Core'!
SystemOrganization addCategory: #'Control-Exceptions'!
SystemOrganization addCategory: #'Control-Support'!

Notification subclass: #DelimitedControlException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Exceptions'!

!DelimitedControlException commentStamp: 'fbs 6/27/2012 20:13' prior: 0!
I mark a context so that a shift further up the stack will unwind only to me. In other words, use me to delimit a continuation.!

Notification subclass: #DelimitedDynamicVariableRef
	instanceVariableNames: 'dynVar value'
	classVariableNames: 'Unset'
	poolDictionaries: ''
	category: 'Control-Support'!

!DelimitedDynamicVariableRef commentStamp: 'fbs 6/27/2012 09:47' prior: 0!
A Notification used to find the current value of a delimited dynamic variable. It contains a reference to the variable whose value it wishes to find. See DelimitedDynamicVariable >> #handle: for details.!

!DelimitedDynamicVariableRef class methodsFor: 'instance creation' stamp: 'fbs 6/26/2012 22:44'!
dynVar: aDelimitedDynamicVariable
	^ self new dynVar: aDelimitedDynamicVariable.! !

!DelimitedDynamicVariableRef class methodsFor: 'instance creation' stamp: 'fbs 7/14/2012 21:24'!
dynVar: aDelimitedDynamicVariable newValue: anObject
	^ self new dynVar: aDelimitedDynamicVariable newValue: anObject.! !

!DelimitedDynamicVariableRef class methodsFor: 'class initialization' stamp: 'fbs 7/14/2012 21:26'!
initialize
	Unset := Object new.! !

!DelimitedDynamicVariableRef methodsFor: 'accessing' stamp: 'fbs 6/26/2012 22:43'!
dynVar
	^ dynVar.! !

!DelimitedDynamicVariableRef methodsFor: 'initialize-release' stamp: 'fbs 7/14/2012 21:26'!
dynVar: aDelimitedDynamicVariable
	self initialize.
	dynVar := aDelimitedDynamicVariable.
	value := Unset.! !

!DelimitedDynamicVariableRef methodsFor: 'initialize-release' stamp: 'fbs 7/14/2012 22:22'!
dynVar: aDelimitedDynamicVariable newValue: anObject
	self initialize.
	dynVar := aDelimitedDynamicVariable.
	value := anObject.! !

!DelimitedDynamicVariableRef methodsFor: 'testing' stamp: 'fbs 7/14/2012 21:27'!
hasValue
	^ value ~= Unset.! !

!DelimitedDynamicVariableRef methodsFor: 'accessing' stamp: 'fbs 7/14/2012 22:22'!
value
	^ value.! !

Notification subclass: #PartialContinuationNotification
	instanceVariableNames: 'topContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Exceptions'!

!PartialContinuationNotification commentStamp: 'ul 4/23/2011 03:29' prior: 0!
Signal me when you want to create a partial continuation. I'm an alternative to the shift/reset methods.
Example code:

c := [ [ 1 + PartialContinuationNotification signal ] value ]
	on: PartialContinuationNotification
	do: [ :notification | notification continuation ].
c value: 5!

!PartialContinuationNotification class methodsFor: 'exceptionInstantiator' stamp: 'ul 4/23/2011 01:03'!
signal

	^self new
		topContext: thisContext sender;
		signal! !

!PartialContinuationNotification methodsFor: 'accessing' stamp: 'ul 4/23/2011 03:11'!
continuation

	^PartialContinuation from: topContext downTo: thisContext sender home! !

!PartialContinuationNotification methodsFor: 'accessing' stamp: 'ul 4/23/2011 01:02'!
topContext: aContextPart

	topContext := aContextPart! !

!BlockClosure methodsFor: '*Control' stamp: 'fbs 9/18/2011 10:54'!
compose: aUnaryBlock
	^ [:x | self value: (aUnaryBlock value: x)].! !

!BlockClosure methodsFor: '*Control' stamp: 'fbs 6/27/2011 22:06'!
reset
	"Mark the call stack. The handler says 'Exception, this is where the nearest reset is."
	^ self on: DelimitedControlException do: [:e | e resume: thisContext home].! !

!BlockClosure methodsFor: '*Control' stamp: 'fbs 6/25/2011 14:04'!
shift
	"I am a unary block. Take the call stack to the nearest reset, reifying that partial continuation as a function F. Then evaluate myself with F as an argument.
	Note that this is a convenience method for BlockClosure>>shift:. See that method's comments for details."
	^ self shift: self.
! !

!BlockClosure methodsFor: '*Control' stamp: 'fbs 6/28/2011 11:12'!
shift: aUnaryBlock
	"Cut contexts off the call stack down to (but not including) the nearest #reset, and reify this partial continuation as a function. Then, pass this function to aUnaryBlock."

	| cc currentCtxt fun resetCtxt |
	currentCtxt := thisContext.
	resetCtxt := DelimitedControlException signal.
	resetCtxt ifNil: [^ MissingResetException signal].
	"resetCtxt is not where we want to cut. Because we use #on:do: to mark the stack, rather than marking the reset's context directly, it's the marked context that we want to leave on the stack. We need to cut to the context whose sender is resetCtxt."
	
	"This search is not safe in the absence of a reset!!"
	cc := currentCtxt.
	(cc sender == resetCtxt) ifFalse:
		[[cc sender sender == resetCtxt] whileFalse: [cc := cc sender]].

	"Store the stack from _below_ the shift to just _below_ the reset marker."
	fun := PartialContinuation from: currentCtxt sender downTo: cc sender.

	"Throw away the stack down to just _above_ the reset marker."
	currentCtxt swapSender: cc sender.

	^ aUnaryBlock value: fun.! !

Object subclass: #DelimitedDynamicVariable
	instanceVariableNames: 'default'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Core'!

!DelimitedDynamicVariable commentStamp: 'fbs 7/14/2012 21:33' prior: 0!
Usually, dynamic variables (such as DynamicVariable) permit either closing over all or none of that variable's changes, causing unexpected and buggy interactions with partial continuations. For example, in the function returned by the #shift in the below:

    TestDynamicVariable value: 1 during: [
        [TestDynamicVariable value: 2 during: [
            [:k| TestDynamicVariable value ] shift ]] reset]

we expect TestDynamicVariable's value to be 1. After all, we have cut out from the stack that part that sets TestDynamicVariable's value to 2. Unfortunately, TestDynamicVariable value = 2, because DynamicVariable is a sequence of do/undo operations not stored directly on the stack.

A DelimitedDynamicVariable, on the other hand, permits #shift to close over PART of a dynamic variable's extent.

For a deep investigation into the bad interactions between full dynamic variables and delimited continuations, see http://okmij.org/ftp/Computation/dynamic-binding.html#DDBinding!

!DelimitedDynamicVariable class methodsFor: 'as yet unclassified' stamp: 'fbs 6/26/2012 15:13'!
default: anObject
	^ self new default: anObject.! !

!DelimitedDynamicVariable methodsFor: 'composing' stamp: 'fbs 7/27/2012 09:21'!
, aDelimitedDynamicVariable
	^ DelimitedDynamicVariableSet with: self with: aDelimitedDynamicVariable! !

!DelimitedDynamicVariable methodsFor: 'accessing' stamp: 'fbs 6/26/2012 19:39'!
default
	^ default.! !

!DelimitedDynamicVariable methodsFor: 'initialize-release' stamp: 'fbs 6/26/2012 15:13'!
default: anObject
	default := anObject! !

!DelimitedDynamicVariable methodsFor: 'controlling' stamp: 'fbs 7/14/2012 21:32'!
dlet: anObject in: aBlock
	"I know which exceptions to handle. See #handle: for details."
	| currentValue |
	"A _mutable_ thing over which we can close. We may not store into a method's arguments."
	currentValue := anObject.
	^ [aBlock value] on: self do: [:e |
		"If the exception has a value set, something has invoked #dset: and thus wishes to replace my current value in this scope."
		e hasValue ifTrue: [currentValue := e value].
		e resume: currentValue]! !

!DelimitedDynamicVariable methodsFor: 'accessing' stamp: 'fbs 6/26/2012 22:44'!
dref
	^ (DelimitedDynamicVariableRef dynVar: self) signal ifNil: [default]! !

!DelimitedDynamicVariable methodsFor: 'controlling' stamp: 'fbs 7/14/2012 22:19'!
dset: anObject
	^ (DelimitedDynamicVariableRef dynVar: self newValue: anObject) signal
		ifNil: [ | oldValue |
			oldValue := default.
			default := anObject. oldValue ].! !

!DelimitedDynamicVariable methodsFor: 'error handling' stamp: 'fbs 6/27/2012 09:49'!
handles: anException
	"In order for us to have one kind of exception for dereferencing a dynamic variable, we need to 'parameterise' the handler. I will only handle the given exception if that exception has been associated with me."
	^ (anException isKindOf: DelimitedDynamicVariableRef)
		and: [anException dynVar == self]! !

Object subclass: #DelimitedDynamicVariableSet
	instanceVariableNames: 'variables'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Core'!

!DelimitedDynamicVariableSet class methodsFor: 'instance creation' stamp: 'fbs 7/27/2012 09:22'!
with: dynVar1 with: dynVar2
	^ self new variables: {dynVar1. dynVar2}.! !

!DelimitedDynamicVariableSet methodsFor: 'controlling' stamp: 'fbs 8/2/2012 18:41'!
dlet: someValues in: aBlock
	"I know which exceptions to handle. See #handle: for details."
	| currentValues |
	variables size = someValues size ifFalse: [
		Error signal: ('Cannot set {1} dynamic variables with {2} values' format: {variables size. someValues size})].
	"A _mutable_ thing over which we can close. We may not store into a method's arguments."
	currentValues := someValues copy.
	^ [aBlock value] on: self do: [:e |
		"If the exception has a value set, something has invoked #dset: and thus wishes to replace my current value in this scope."
		e hasValue ifTrue: [currentValues at: (variables indexOf: e dynVar) put: e value].
		e resume: (currentValues at: (variables indexOf: e dynVar))]! !

!DelimitedDynamicVariableSet methodsFor: 'error handling' stamp: 'fbs 7/27/2012 09:30'!
handles: anException
	"In order for us to have one kind of exception for dereferencing a dynamic variable, we need to 'parameterise' the handler. I will only handle the given exception if that exception has been associated with me."
	^ (anException isKindOf: DelimitedDynamicVariableRef)
		and: [variables includes: anException dynVar]! !

!DelimitedDynamicVariableSet methodsFor: 'initialize-release' stamp: 'fbs 7/27/2012 09:22'!
variables: someDynVars
	variables := someDynVars.! !

!Object methodsFor: '*Control' stamp: 'fbs 6/22/2011 20:19'!
asContinuation
	^ PartialContinuation return: self.! !

Object subclass: #PartialContinuation
	instanceVariableNames: 'context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Core'!

!PartialContinuation commentStamp: 'ul 4/28/2011 00:13' prior: 0!
I am a slice of a call stack, that is to say, some sequence of ContextParts. Use me like a Block by calling my #value or #value: methods.!

!PartialContinuation class methodsFor: 'instance creation' stamp: 'fbs 4/12/2011 19:11'!
from: topContext downTo: bottomContext
	^ self new from: topContext downTo: bottomContext.! !

!PartialContinuation class methodsFor: 'monad' stamp: 'fbs 6/23/2012 20:51'!
return: aBlocklikeObject
	^ [[:k | k value: [:x | aBlocklikeObject value: x] ] shift ] reset.! !

!PartialContinuation methodsFor: 'monad' stamp: 'fbs 6/24/2011 07:26'!
>>= aContinuation
	"Add aContinuation to the continuation chain."
	^ aContinuation compose: self.! !

!PartialContinuation methodsFor: 'evaluating' stamp: 'fbs 6/21/2011 22:10'!
compose: aUnaryBlock
	^ [:x | self value: (aUnaryBlock value: x)].! !

!PartialContinuation methodsFor: 'debugging' stamp: 'fbs 6/20/2011 17:54'!
debugString
	| delim s |
	delim := Character cr asString, '=======', Character cr.
	s := self stack.
	^ s allButFirst inject: (s first method decompileString), delim into: [:acc :each | acc, each method decompileString, delim].! !

!PartialContinuation methodsFor: 'initialize-release' stamp: 'ul 4/23/2011 23:06'!
from: topContext downTo: bottomContext

	context := topContext copyTo: bottomContext! !

!PartialContinuation methodsFor: 'accessing' stamp: 'fbs 4/14/2011 15:41'!
numArgs
	^ 1! !

!PartialContinuation methodsFor: 'accessing' stamp: 'ul 4/23/2011 23:07'!
stack
	"A debugging aid. Return a Collection containing my contexts."

	| result currentContext |
	result := OrderedCollection new.
	currentContext := context.
	[ currentContext isNil ] whileFalse: [
		result addLast: currentContext.
		currentContext := currentContext sender ].
	^result! !

!PartialContinuation methodsFor: 'evaluating' stamp: 'fbs 4/12/2011 18:59'!
value
	"Splice my contents onto the stack, and execute me."
	self value: nil.! !

!PartialContinuation methodsFor: 'evaluating' stamp: 'ul 4/28/2011 03:03'!
value: anObject
	"Copy the contexts of the continuation and stitch them onto the current call stack by telling the bottom context that thisContext's sender called it. 'thisContext sender' because thisContext refers to the context of the #value: call."
	
	| topContext bottomContext |
	topContext := context  
		copyTo: nil "copy the whole stack"
		bottomContextDo: [ :bc | bottomContext := bc ].
	bottomContext swapSender: thisContext sender.
	"Tell thisContext that its sender was actually the last/top-most context."
	thisContext swapSender: topContext.
	"Finally, 'return' anObject from the caller context... which thanks to the line
	above, is actually values first. This effectively sets anObject as the parameter
	to the reified partial continuation, because instead of 'returning' to our caller
	we 'return' to the next part of the computation - our
	partial-continuation-as-function."
	^anObject.! !

Error subclass: #MissingResetException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Exceptions'!
DelimitedDynamicVariableRef initialize!
